@@ -5,10 +5,6 @@
 #include <stdlib.h>
 #include <ncurses.h>
 #include "eisl.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
-#include "mem.h"
 
 #define TOKEN_MAX 80
 #define FRAGMENT_MAX 20
@@ -33,9 +29,9 @@ int f_edit(int arglist, int th)
 	    ed = "edlis";
 	}
     }
-    char *str = Str_catv(ed, 1, 0, " ", 1, 0, GET_NAME(arg1), 1, 0, NULL);
+    char *str = eisl_strcatv(ed, " ", GET_NAME(arg1), (const char *) NULL);
     res = system(str);
-    FREE(str);
+    free(str);
     if (res == -1)
 	error(SYSTEM_ERR, "edit", arg1, th);
     f_load(arglist, 0);
@@ -850,10 +846,10 @@ read_line_loop(int c, int *j, int *uni_j, int *pos, int limit,
 		    for (i = 0; i < CANDIDATE; i++) {
 			if (i + k >= ed_candidate_pt)
 			    break;
-			Fmt_print("%d:%s ", i + 1, ed_candidate[i + k]);
+			printf("%d:%s ", i + 1, ed_candidate[i + k]);
 		    }
 		    if (ed_candidate_pt > k + CANDIDATE)
-			Fmt_print("4:more");
+			printf("4:more");
 		    ESCRST();
 		    bool bad_candidate_selected;
 		    do {
@@ -892,7 +888,7 @@ read_line_loop(int c, int *j, int *uni_j, int *pos, int limit,
 	    return false;
 	case 'q':		// Esc+q
 	    putchar('\n');
-	    RAISE(Exit_Interp);
+	    longjmp(repl_buf, 2);
 	    break;
 	case ARROW_PREFIX:
 	    c = eisl_getch();
