@@ -1739,7 +1739,7 @@ char *getname()
 	    cancel_flag = true;
 	    return (buf);
 	default:
-	    if (pos > SHORT_STR_MAX)
+	    if (pos >= SHORT_STR_MAX)
 		break;
 	    else if (c < 20)
 		break;
@@ -1794,7 +1794,7 @@ char *getword1()
 	    cancel_flag = true;
 	    return (buf);
 	default:
-	    if (pos > SHORT_STR_MAX)
+	    if (pos >= SHORT_STR_MAX)
 		break;
 	    else if (c < 20)
 		break;
@@ -1848,7 +1848,7 @@ char *getword2()
 	    cancel_flag = true;
 	    return (buf);
 	default:
-	    if (pos > SHORT_STR_MAX)
+	    if (pos >= SHORT_STR_MAX)
 		break;
 	    else if (c < 20)
 		break;
@@ -2252,7 +2252,7 @@ void help(void)
 */
 void display_unicode(int line, int col)
 {
-    char mb[10];
+    char mb[10] = { 0 };
     wchar_t wch[10];
     if (isUni1(ed_data[line][col])) {
 	mb[0] = ed_data[line][col];
@@ -2593,6 +2593,8 @@ int find_eol(int row)
 {
     int i;
 
+    if (row < 0)
+	return (-1);
     for (i = 0; i < COL_SIZE; i++) {
 	if (ed_data[row][i] == EOL || ed_data[row][i] == 0)
 	    return (i);
@@ -2606,6 +2608,9 @@ int find_eol(int row)
 int find_eol1(int row)
 {
     int col, col1;		// col1 is position of display terminal
+
+    if (row < 0)
+	return (-1);
     col = col1 = 0;
     while (col < COL_SIZE) {
 	if (ed_data[row][col] == EOL)
