@@ -3,13 +3,12 @@
 
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 700
-#endif	/* 
- */
+#endif
     
 #define FULLSCREEN
 #include "term.h"
 
-static const float VERSION = 5.50;
+static const float VERSION = 5.61;
 
 #define ROW_SIZE 5000
 #define COL_SIZE 256
