@@ -1,5 +1,5 @@
 /*
- * Easy-ISLisp (ISLisp) written by kenichi sasagawa 2016/4 ~ 2023/9
+ * Easy-ISLisp (ISLisp) written by kenichi sasagawa 2016/4 ~ 
  */
 
 #ifndef EISL_H
@@ -18,7 +18,6 @@
 #include "compat/cdefs.h"
 #include "ffi.h"
 #include "term.h"
-#include "except.h"
 #include "compat/eiffel_stubs.h"
 
 
@@ -84,7 +83,7 @@
 
 
 
-static const float VERSION = 5.58;
+static const float VERSION = 5.61;
 static const int FREESIZE = 1000000;
 static const int SYMSIZE = 256;
 static const int CHARSIZE = 7;	// unicode char. add \0 to tail
@@ -511,9 +510,18 @@ static inline int IDX2R(int i, int j, int ld)
 static inline void append_str(int output_stream, const char *from)
 {
     char *to = GET_NAME(output_stream);
-    strncat(to, from, STRSIZE - strlen(to) - 1);
-    to[STRSIZE - 1] = '\0';
-    SET_PROP(output_stream, GET_PROP(output_stream) + strlen(from));
+    size_t len_to = strlen(to);
+    size_t len_from = strlen(from);
+
+    size_t remain = STRSIZE - len_to - 1;
+
+    if (remain > 0) {
+        size_t copy_len = (len_from < remain) ? len_from : remain;
+        memcpy(to + len_to, from, copy_len);
+        to[len_to + copy_len] = '\0';
+    }
+
+    SET_PROP(output_stream, GET_PROP(output_stream) + len_from);
 }
 
 static inline void output_str(int output_stream, const char *from)
@@ -587,14 +595,14 @@ extern double try_timer;
 extern int try_res;
 
 /* longjmp control and etc */
-extern Except_T Restart_Repl, Exit_Interp;
 extern jmp_buf block_buf[CTRLSTK];
 extern jmp_buf catch_buf[CTRLSTK];
 extern jmp_buf cont_buf;
-extern Except_T Ignored_Error;
-extern Except_T Exit_Thread;
-extern Except_T Exit_Process;
-extern Except_T Exit_Network;
+extern jmp_buf error_buf;
+extern jmp_buf repl_buf;
+extern jmp_buf thread_buf;
+extern jmp_buf process_buf;
+extern jmp_buf network_buf;
 extern int block_tag_check[CTRLSTK];
 extern int block_data[CTRLSTK][3];
 extern int catch_data[CTRLSTK][4];
@@ -988,6 +996,13 @@ int f_get_universal_time(int arglist, int th);
 int f_go(int arglist, int th);
 int f_getenv(int arglist, int th);
 int f_greater(int arglist, int th);
+int f_gr_open(int arglist, int th);
+int f_gr_close(int arglist, int th);
+int f_gr_cls(int arglist, int th);
+int f_gr_pset(int arglit, int th);
+int f_gr_circle(int arglist, int th);
+int f_gr_line(int arglist, int th);
+int f_gr_rect(int arglist, int th);
 int f_heapdump(int arglist, int th);
 int f_identity(int arglist, int th);
 int f_if(int arglist, int th);
@@ -1553,6 +1568,10 @@ int set_catch_symbols(int x);
 char *get_name(int x);
 double get_flt(int x);
 int jump_to_repl();
+char *eisl_strdup(const char *s);
+char *eisl_strcatv(const char *first, ...);
+char *eisl_strcat(const char *s1, const char *s2);
+const char *eisl_strchr(const char *str, char ch);
 
 /* Edlis */
 void display_buffer(void);
