@@ -4,10 +4,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include "eisl.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
-#include "mem.h"
 
 static int outc(int c)
 {
@@ -634,29 +630,29 @@ int signal_condition(int x, int y, int th)
 	SET_OPT(x, NOTCONT);
     else {
 	SET_OPT(x, CONTINUABLE);
-	TRY heap[x].name = Str_dup(GET_NAME(y), 1, 0, 1);
-	EXCEPT(Mem_Failed) error(MALLOC_OVERF, "signal-condition", NIL,
-				 th);
-	END_TRY;
+	heap[x].name = eisl_strdup(GET_NAME(y));
+	if (heap[x].name == NULL) {
+	    error(MALLOC_OVERF, "signal-condition", NIL, th);
+	}
     }
     if (ignore_flag) {
 	block_pt = 0;
 	handling_resource_err = false;
-	RAISE(Ignored_Error);
+	longjmp(error_buf, 1);
     }
     if (parallel_flag) {
 	handling_resource_err = false;
 	signal_condition_x = x;
 	signal_condition_y = y;
-	RAISE(Exit_Thread);
+	longjmp(thread_buf, 1);
     }
     if (process_flag) {
 	handling_resource_err = false;
 	signal_condition_x = x;
 	signal_condition_y = y;
-	Fmt_print("\x15");
+	printf("\x15");
 	fflush(stdout);
-	RAISE(Exit_Process);
+	longjmp(process_buf, 1);
     }
     if (child_flag) {
 	handling_resource_err = false;
@@ -684,7 +680,7 @@ int signal_condition(int x, int y, int th)
     if (open_flag && error_handler == NIL) {
 	fclose(GET_PORT(input_stream));
 	open_flag = false;
-	Fmt_print("around here line=%d column=%d\n", line, column);
+	printf("around here line=%d column=%d\n", line, column);
     }
     if (error_handler != NIL) {
 	int handler;
@@ -738,7 +734,7 @@ int signal_condition(int x, int y, int th)
 	puts("Error occurred while executing command line options.");
 	option_flag = false;
     }
-    RAISE(Restart_Repl);
+    longjmp(repl_buf, 1);
     return 0;
 }
 
