@@ -9,6 +9,7 @@
  * functions for multi-process
  * functions for distributed-process
  * functions for TCP/IP
+ * functions for simple graphics
  */
 
 #include <stdio.h>
@@ -19,9 +20,13 @@
 #include <errno.h>
 #include <pthread.h>
 #include <sys/time.h>
+#include <sys/mman.h>
+#include <linux/fb.h>
+#include <linux/fb.h>
+#include <sys/ioctl.h>
 
 
-#ifdef __rpi__
+#ifdef __rpiwiring__
 #include <wiringPi.h>
 #include <wiringPiSPI.h>
 #include <arpa/inet.h>
@@ -33,11 +38,7 @@
 #endif
 
 #include "eisl.h"
-#include "mem.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
-#include "text.h"
+
 
 void init_exsubr(void)
 {
@@ -127,7 +128,15 @@ void init_exsubr(void)
     def_subr("RECV-SOCKET", f_recv_socket);
     def_subr("CLOSE-SOCKET", f_close_socket);
 
-#ifdef __rpi__
+    def_subr("GR-OPEN", f_gr_open);
+    def_subr("GR-CLOSE", f_gr_close);
+    def_subr("GR-CLS", f_gr_cls);
+    def_subr("GR-PSET", f_gr_pset);
+    def_subr("GR-CIRCLE", f_gr_circle);
+    def_subr("GR-RECT", f_gr_rect);
+    def_subr("GR-LINE", f_gr_line);
+
+#ifdef __rpiwiring__
     def_subr("WIRINGPI-SETUP-GPIO", f_wiringpi_setup_gpio);
     def_subr("WIRINGPI-SPI-SETUP-CH-SPEED", f_wiringpi_spi_setup_ch_speed);
     def_subr("PWM-SET-MODE", f_pwm_set_mode);
@@ -562,9 +571,10 @@ int f_set_random(int arglist, int th)
 	error(NOT_NUM, "set-random", arg1, th);
 
     n = GET_INT(arg1);
-    if (n < 0)
+    if (n < 0) {
 	error(ILLEGAL_ARGS, "set-random", n, th);
-
+	return (0);
+    }
     srand(n);
     return (arg1);
 }
@@ -666,7 +676,7 @@ void debugger(int th)
 		 ":q quit\n"
 		 ":r room\n" ":s stepper ON/OFF\n" "other S exps eval");
 	} else if (eqp(x, make_sym(":A"))) {
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	} else if (eqp(x, make_sym(":B"))) {
 	    for (i = 0; i < BACKSIZE; i++) {
 		print(backtrace[i]);
@@ -674,7 +684,7 @@ void debugger(int th)
 	    }
 	} else if (eqp(x, make_sym(":D"))) {
 	    for (i = 0; i < thread_num; i++) {
-		Fmt_print("thread%d = ", i);
+		printf("thread%d = ", i);
 		for (j = 0; j < dp[i]; j++) {
 		    print(dynamic[j][0][i]);
 		    printf("=");
@@ -685,7 +695,7 @@ void debugger(int th)
 	    }
 	} else if (eqp(x, make_sym(":E"))) {
 	    for (i = 0; i < thread_num; i++) {
-		Fmt_print("thread%d = ", i);
+		printf("thread%d = ", i);
 		print(ep[i]);
 		putchar('\n');
 	    }
@@ -695,19 +705,19 @@ void debugger(int th)
 	} else if (eqp(x, make_sym(":Q"))) {
 	    return;
 	} else if (eqp(x, make_sym(":R"))) {
-	    Fmt_print("EP = %d (environment pointer)\n"
-		      "DP = %d (dynamic pointer)\n"
-		      "HP = %d (heap pointer)\n"
-		      "SP = %d (stack pointer)\n"
-		      "AP = %d (arglist pointer)\n"
-		      "LP = %d (shelter pointer)\n"
-		      "Parallel = %d (thread_num)\n"
-		      "Thread = %d (current thread)\n",
-		      ep[th], dp[th], hp[th], sp[th], ap[th],
-		      lp[th], thread_num, th);
+	    printf("EP = %d (environment pointer)\n"
+		   "DP = %d (dynamic pointer)\n"
+		   "HP = %d (heap pointer)\n"
+		   "SP = %d (stack pointer)\n"
+		   "AP = %d (arglist pointer)\n"
+		   "LP = %d (shelter pointer)\n"
+		   "Parallel = %d (thread_num)\n"
+		   "Thread = %d (current thread)\n",
+		   ep[th], dp[th], hp[th], sp[th], ap[th],
+		   lp[th], thread_num, th);
 	    puts("Free cell ");
 	    for (i = 0; i < thread_num; i++) {
-		Fmt_print("thread%d = %d\n", i, fc[i]);
+		printf("thread%d = %d\n", i, fc[i]);
 	    }
 	} else if (eqp(x, make_sym(":S"))) {
 	    if (stepper_flag == 0) {
@@ -759,55 +769,55 @@ void cellprint(int addr)
 	puts("EMP");
 	break;
     case INTN:
-	Fmt_print("INTN   %d\n", GET_INT(addr));
+	printf("INTN   %d\n", GET_INT(addr));
 	break;
     case FLTN:
-	Fmt_print("FLTN   %f\n", GET_FLT(addr));
+	printf("FLTN   %f\n", GET_FLT(addr));
 	break;
     case LONGN:
-	Fmt_print("LONGN  %D\n", GET_LONG(addr));
+	printf("LONGN  %lld\n", GET_LONG(addr));
 	break;
     case BIGN:
-	Fmt_print("BIGN   %d\n", GET_CAR(addr));
+	printf("BIGN   %d\n", GET_CAR(addr));
 	break;
     case SYM:
-	Fmt_print("SYM    %07d %07d %07d %s\n", GET_CAR(addr),
-		  GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
+	printf("SYM    %07d %07d %07d %s\n", GET_CAR(addr),
+	       GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
 	break;
     case STR:
-	Fmt_print("STR    %07d %07d %07d %s\n", GET_CAR(addr),
-		  GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
+	printf("STR    %07d %07d %07d %s\n", GET_CAR(addr),
+	       GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
 	break;
     case LIS:
-	Fmt_print("LIS    %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("LIS    %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     case SUBR:
-	Fmt_print("SUBR   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("SUBR   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     case FSUBR:
-	Fmt_print("FSUBR  %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("FSUBR  %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     case FUNC:
-	Fmt_print("FUNC   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("FUNC   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     case MACRO:
-	Fmt_print("MACRO  %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("MACRO  %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     case CLASS:
-	Fmt_print("CLASS  %07d %07d %07d %s\n", GET_CAR(addr),
-		  GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
+	printf("CLASS  %07d %07d %07d %s\n", GET_CAR(addr),
+	       GET_CDR(addr), GET_AUX(addr), GET_NAME(addr));
 	break;
     case GENERIC:
-	Fmt_print("GENE   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
-		  GET_AUX(addr));
+	printf("GENE   %07d %07d %07d\n", GET_CAR(addr), GET_CDR(addr),
+	       GET_AUX(addr));
 	break;
     default:
-	Fmt_print("cellprint(%d) tag switch default action\n", addr);
+	printf("cellprint(%d) tag switch default action\n", addr);
     }
 }
 
@@ -818,7 +828,7 @@ void heapdump(int start, int end)
 
     puts("addr    F   TAG    CAR     CDR     AUX     NAME");
     for (i = start; i <= end; i++) {
-	Fmt_print("%07d ", i);
+	printf("%07d ", i);
 	cellprint(i);
     }
 }
@@ -960,7 +970,7 @@ int f_get_myself(int arglist, int th)
 /* for Raspberry PI 
  * wiringpi for GPIO 
  */
-#ifdef __rpi__
+#ifdef __rpiwiring__
 int f_wiringpi_setup_gpio(int arglist __unused, int th __unused)
 {
     wiringPiSetupGpio();
@@ -1387,7 +1397,7 @@ int f_time(int arglist, int th)
     st = getETime();
     eval(arg1, 0);
     en = getETime();
-    Fmt_print("Elapsed Time(second)=%.6f\n", en - st);
+    printf("Elapsed Time(second)=%.6f\n", en - st);
     return (UNDEF);
 }
 
@@ -1440,28 +1450,28 @@ int f_import(int arglist, int th)
     if (!stringp(arg1))
 	error(NOT_SYM, "import", arg1, th);
 
-    char *str = Str_cat(GET_NAME(arg1), 1, 0, ".o", 1, 0);
+    char *str = eisl_strcat(GET_NAME(arg1), ".o");
     char *fname = library_file(str);
     if (access(fname, R_OK) != -1) {
 	f_load(list1(make_str(fname)), 0);
 	goto cleanup;
     }
 
-    FREE(str);
-    str = Str_cat(GET_NAME(arg1), 1, 0, ".lsp", 1, 0);
-    FREE(fname);
+    free(str);
+    str = eisl_strcat(GET_NAME(arg1), ".lsp");
+    free(fname);
     fname = library_file(str);
     if (access(fname, R_OK) != -1) {
 	f_load(list1(make_str(fname)), 0);
 	goto cleanup;
     }
-    FREE(str);
-    FREE(fname);
+    free(str);
+    free(fname);
     error(CANT_OPEN, "import", arg1, th);
 
   cleanup:
-    FREE(str);
-    FREE(fname);
+    free(str);
+    free(fname);
     return (T);
 }
 
@@ -1593,7 +1603,7 @@ int modulesubst1(int x, int module)
 {
     char str[SYMSIZE];
 
-    Fmt_sfmt(str, SYMSIZE, "%s::%s", GET_NAME(module), GET_NAME(x));
+    sprintf(str, "%s::%s", GET_NAME(module), GET_NAME(x));
     return (make_sym(str));
 }
 
@@ -1629,7 +1639,7 @@ int program;
 
 int f_try(int arglist, int th)
 {
-    int arg1, arg2, arg3, arg4, pos, c, bit, i, res, save1, save2;
+    int arg1, arg2, arg3, arg4, pos, c, bit, i, res, save1, save2, ret;
     char str[STRSIZE];
 
     arg1 = car(arglist);	//time 
@@ -1671,18 +1681,22 @@ int f_try(int arglist, int th)
     str[pos] = NUL;
 
     program = make_stm(stdin, EISL_INSTR, NULL);
-    TRY heap[program].name = Str_dup(str, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "try", NIL, th);
-    END_TRY;
+    heap[program].name = eisl_strdup(str);
+    if (heap[program].name == NULL) {
+	error(MALLOC_OVERF, "try", NIL, th);
+    }
 
     save1 = input_stream;
     save2 = output_stream;
 
     if (arg1 == make_sym("NO-TIME-LIMIT")) {
 	ignore_flag = true;
-	TRY res = eval(arg2, 0);
-	ELSE res = UNDEF;
-	END_TRY;
+	ret = setjmp(error_buf);
+	if (ret == 0) {
+	    res = eval(arg2, 0);
+	} else if (ret == 1) {
+	    res = UNDEF;
+	}
 	ignore_flag = false;
 	if (res == UNDEF)
 	    res = make_sym("FAILSE");
@@ -1693,9 +1707,12 @@ int f_try(int arglist, int th)
 	try_res = NIL;
 	try_flag = true;
 	ignore_flag = true;
-	TRY res = eval(arg2, th);
-	ELSE res = UNDEF;
-	END_TRY;
+	ret = setjmp(error_buf);
+	if (ret == 0) {
+	    res = eval(arg2, th);
+	} else if (ret == 1) {
+	    res = UNDEF;
+	}
 	ignore_flag = false;
 	try_flag = false;
 	if (res == UNDEF) {
@@ -1726,7 +1743,7 @@ int f_try(int arglist, int th)
 
 int f_read_exp(int arglist, int th)
 {
-    int res, save;
+    int res, save, ret;
 
     if (!nullp(arglist))
 	error(ILLEGAL_ARGS, "read-exp", arglist, th);
@@ -1734,9 +1751,15 @@ int f_read_exp(int arglist, int th)
 
     save = input_stream;
     input_stream = program;
-    TRY res = sread();
-    ELSE res = UNDEF;
-    END_TRY;
+    ignore_flag = true;
+    res = NIL;
+    ret = setjmp(error_buf);
+    if (ret == 0) {
+	res = sread();
+    } else if (ret == 1) {
+	res = UNDEF;
+    }
+    ignore_flag = false;
     input_stream = save;
 
     return (res);
@@ -1761,6 +1784,7 @@ int f_create_client_socket(int arglist, int th)
     sock = socket(AF_INET, SOCK_STREAM, 0);
     if (sock < 0) {
 	error(SYSTEM_ERR, "create-client-socket", NIL, th);
+	return (0);
     }
 
     memset((char *) &client_addr, 0, sizeof(client_addr));
@@ -1794,6 +1818,7 @@ int f_create_server_socket(int arglist, int th)
     sock0 = socket(AF_INET, SOCK_STREAM, 0);
     if (sock0 < 0) {
 	error(SYSTEM_ERR, "cread-server-socket", NIL, th);
+	return (0);
     }
 
     memset((char *) &server_addr, 0, sizeof(server_addr));
@@ -1805,6 +1830,8 @@ int f_create_server_socket(int arglist, int th)
 	(sock0, (struct sockaddr *) &server_addr, sizeof(server_addr)) < 0)
     {
 	error(SYSTEM_ERR, "create-server-socket", NIL, th);
+	close(sock0);
+	return (0);
     }
 
     listen(sock0, 5);
@@ -1814,7 +1841,8 @@ int f_create_server_socket(int arglist, int th)
     sock1 = accept(sock0, (struct sockaddr *) &server_addr, &parent_len);
     if (sock1 < 0) {
 	error(SYSTEM_ERR, "create-server-socket", NIL, th);
-
+	close(sock0);
+	return (0);
     }
 
     res = make_socket(sock1, EISL_SOCKET, "server", sock0);
@@ -1877,3 +1905,355 @@ int f_close_socket(int arglist, int th)
 
     return (T);
 }
+
+
+//-------/dev/fb0------------------------
+
+#ifdef __rpi__
+#define BLACK       0x000000
+#define BLUE        0xFF0000
+#define RED         0x0000FF
+#define MAGENTA     0xFF00FF
+#define GREEN       0x00FF00
+#define CYAN        0xFFFF00
+#define YELLOW      0x00FFFF
+#define WHITE       0xFFFFFF
+
+#else				// RGB888
+
+#define BLACK       0x000000
+#define BLUE        0x0000FF
+#define RED         0xFF0000
+#define MAGENTA     0xFF00FF
+#define GREEN       0x00FF00
+#define CYAN        0x00FFFF
+#define YELLOW      0xFFFF00
+#define WHITE       0xFFFFFF
+
+#endif
+
+
+static int fb = -1;
+static char *fbp = NULL;
+static struct fb_var_screeninfo vinfo;
+static long screensize = 0;
+
+int fb_open()
+{
+    fb = open("/dev/fb0", O_RDWR);
+    if (fb < 0)
+	return -1;
+
+    if (ioctl(fb, FBIOGET_VSCREENINFO, &vinfo)) {
+	close(fb);
+	return -1;
+    }
+
+    screensize =
+	vinfo.yres_virtual * vinfo.xres_virtual * vinfo.bits_per_pixel / 8;
+    fbp = mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED, fb, 0);
+    if (fbp == MAP_FAILED) {
+	close(fb);
+	return -1;
+    }
+    return 0;
+}
+
+void fb_flush()
+{
+    if (fb != -1) {
+	if (ioctl(fb, FBIOPAN_DISPLAY, &vinfo)) {
+	    perror("FBIOPAN_DISPLAY failed");
+	}
+    }
+}
+
+void fb_close()
+{
+    if (fbp)
+	munmap(fbp, screensize);
+    if (fb >= 0)
+	close(fb);
+}
+
+
+void fb_draw_pixel(int x, int y, unsigned int color)
+{
+    if (x < 0 || y < 0 || x >= vinfo.xres_virtual
+	|| y >= vinfo.yres_virtual)
+	return;
+    long location = (x + vinfo.xoffset) * (vinfo.bits_per_pixel / 8) +
+	(y +
+	 vinfo.yoffset) * vinfo.xres_virtual * (vinfo.bits_per_pixel / 8);
+    *((unsigned int *) (fbp + location)) = color;
+
+}
+
+
+void fb_clear_screen(unsigned int color)
+{
+    for (int y = 0; y < vinfo.yres_virtual; y++) {
+	for (int x = 0; x < vinfo.xres_virtual; x++) {
+	    fb_draw_pixel(x, y, color);
+	}
+    }
+}
+
+void fb_draw_circle(int cx, int cy, int r, unsigned int color, int fill)
+{
+    int x = 0;
+    int y = r;
+    int d = 3 - 2 * r;
+
+    while (y >= x) {
+	if (fill) {
+	    // scanline 塗りつぶし
+	    for (int i = cx - x; i <= cx + x; i++) {
+		fb_draw_pixel(i, cy + y, color);
+		fb_draw_pixel(i, cy - y, color);
+	    }
+	    for (int i = cx - y; i <= cx + y; i++) {
+		fb_draw_pixel(i, cy + x, color);
+		fb_draw_pixel(i, cy - x, color);
+	    }
+	} else {
+	    // 円周だけ
+	    fb_draw_pixel(cx + x, cy + y, color);
+	    fb_draw_pixel(cx - x, cy + y, color);
+	    fb_draw_pixel(cx + x, cy - y, color);
+	    fb_draw_pixel(cx - x, cy - y, color);
+	    fb_draw_pixel(cx + y, cy + x, color);
+	    fb_draw_pixel(cx - y, cy + x, color);
+	    fb_draw_pixel(cx + y, cy - x, color);
+	    fb_draw_pixel(cx - y, cy - x, color);
+	}
+
+	if (d <= 0) {
+	    d = d + 4 * x + 6;
+	} else {
+	    d = d + 4 * (x - y) + 10;
+	    y--;
+	}
+	x++;
+    }
+}
+
+
+void fb_draw_rect(int x0, int y0, int x1, int y1, unsigned int color,
+		  int fill)
+{
+    if (x0 > x1) {
+	int t = x0;
+	x0 = x1;
+	x1 = t;
+    }
+    if (y0 > y1) {
+	int t = y0;
+	y0 = y1;
+	y1 = t;
+    }
+
+    if (fill) {
+	for (int y = y0; y <= y1; y++) {
+	    for (int x = x0; x <= x1; x++) {
+		fb_draw_pixel(x, y, color);
+	    }
+	}
+    } else {
+	for (int x = x0; x <= x1; x++) {
+	    fb_draw_pixel(x, y0, color);
+	    fb_draw_pixel(x, y1, color);
+	}
+	for (int y = y0 + 1; y < y1; y++) {
+	    fb_draw_pixel(x0, y, color);
+	    fb_draw_pixel(x1, y, color);
+	}
+    }
+}
+
+void fb_draw_line(int x0, int y0, int x1, int y1, unsigned int color)
+{
+    int dx = abs(x1 - x0);
+    int sx = x0 < x1 ? 1 : -1;
+    int dy = -abs(y1 - y0);
+    int sy = y0 < y1 ? 1 : -1;
+    int err = dx + dy;
+    while (1) {
+	fb_draw_pixel(x0, y0, color);
+	if (x0 == x1 && y0 == y1)
+	    break;
+	int e2 = 2 * err;
+	if (e2 >= dy) {
+	    err += dy;
+	    x0 += sx;
+	}
+	if (e2 <= dx) {
+	    err += dx;
+	    y0 += sy;
+	}
+    }
+}
+
+int color_to_number(int symbol)
+{
+    if (eqp(symbol, make_sym("BLACK")))
+	return (BLACK);
+    else if (eqp(symbol, make_sym("BLUE")))
+	return (BLUE);
+    else if (eqp(symbol, make_sym("RED")))
+	return (RED);
+    else if (eqp(symbol, make_sym("MAGENTA")))
+	return (MAGENTA);
+    else if (eqp(symbol, make_sym("GREEN")))
+	return (GREEN);
+    else if (eqp(symbol, make_sym("CYAN")))
+	return (CYAN);
+    else if (eqp(symbol, make_sym("YELLOW")))
+	return (YELLOW);
+    else if (eqp(symbol, make_sym("WHITE")))
+	return (WHITE);
+
+    return (0);
+}
+
+int f_gr_open(int arglist, int th)
+{
+    int res;
+    if (!nullp(arglist))
+	error(WRONG_ARGS, "GR-OPEN", arglist, th);
+
+    res = fb_open();
+    if (res == 0)
+	return (T);
+    else if (res == -1)
+	return (NIL);
+
+    return (NIL);
+}
+
+int f_gr_close(int arglist, int th)
+{
+    if (!nullp(arglist))
+	error(WRONG_ARGS, "GR-CLOSE", arglist, th);
+
+    fb_close();
+    return (T);
+}
+
+int f_gr_cls(int arglist, int th)
+{
+    int arg1;
+    arg1 = car(arglist);
+    if (!symbolp(arg1))
+	error(NOT_SYM, "GR-CLS", arg1, th);
+
+    fb_clear_screen(color_to_number(arg1));
+    return (T);
+}
+
+
+
+int f_gr_pset(int arglist, int th)
+{
+    int arg1, arg2, arg3;
+    arg1 = car(arglist);
+    arg2 = cadr(arglist);
+    arg3 = caddr(arglist);
+    if (!integerp(arg1))
+	error(NOT_INT, "GR-PSET", arg1, th);
+    if (!integerp(arg2))
+	error(NOT_INT, "GR-PSET", arg2, th);
+    if (!symbolp(arg3))
+	error(NOT_SYM, "GR-PSET", arg3, th);
+
+    fb_draw_pixel(GET_INT(arg1), GET_INT(arg2), color_to_number(arg3));
+    return (T);
+}
+
+
+
+
+int f_gr_circle(int arglist, int th)
+{
+    int arg1, arg2, arg3, arg4, arg5;
+    arg1 = car(arglist);
+    arg2 = cadr(arglist);
+    arg3 = caddr(arglist);
+    arg4 = car(cdddr(arglist));
+    arg5 = car(cdr(cdddr(arglist)));
+    if (!integerp(arg1))
+	error(NOT_INT, "GR-CIRCLE", arg1, th);
+    if (!integerp(arg2))
+	error(NOT_INT, "GR-CIRCLE", arg2, th);
+    if (!integerp(arg3))
+	error(NOT_INT, "GR-CIRCLE", arg3, th);
+    if (!symbolp(arg4))
+	error(NOT_SYM, "GR-CIRCLE", arg4, th);
+
+    if (nullp(arg5))
+	fb_draw_circle(GET_INT(arg1), GET_INT(arg2), GET_INT(arg3),
+		       color_to_number(arg4), 0);
+    else if (eqp(arg5, make_sym("FILL")))
+	fb_draw_circle(GET_INT(arg1), GET_INT(arg2), GET_INT(arg3),
+		       color_to_number(arg4), 1);
+
+    return (T);
+}
+
+
+int f_gr_rect(int arglist, int th)
+{
+    int arg1, arg2, arg3, arg4, arg5, arg6;
+    arg1 = car(arglist);
+    arg2 = cadr(arglist);
+    arg3 = caddr(arglist);
+    arg4 = car(cdddr(arglist));
+    arg5 = car(cdr(cdddr(arglist)));
+    arg6 = car(cddr(cdddr(arglist)));
+
+    if (!integerp(arg1))
+	error(NOT_INT, "GR-RECT", arg1, th);
+    if (!integerp(arg2))
+	error(NOT_INT, "GR-RECT", arg2, th);
+    if (!integerp(arg3))
+	error(NOT_INT, "GR-RECT", arg3, th);
+    if (!integerp(arg4))
+	error(NOT_INT, "GR-RECT", arg4, th);
+    if (!symbolp(arg5))
+	error(NOT_SYM, "GR-RECT", arg5, th);
+
+    if (nullp(arg6))
+	fb_draw_rect(GET_INT(arg1), GET_INT(arg2), GET_INT(arg3),
+		     GET_INT(arg4), color_to_number(arg5), 0);
+    else if (eqp(arg6, make_sym("FILL")))
+	fb_draw_rect(GET_INT(arg1), GET_INT(arg2), GET_INT(arg3),
+		     GET_INT(arg4), color_to_number(arg5), 1);
+
+    return (T);
+}
+
+
+
+int f_gr_line(int arglist, int th)
+{
+    int arg1, arg2, arg3, arg4, arg5;
+    arg1 = car(arglist);
+    arg2 = cadr(arglist);
+    arg3 = caddr(arglist);
+    arg4 = car(cdddr(arglist));
+    arg5 = car(cdr(cdddr(arglist)));
+    if (!integerp(arg1))
+	error(NOT_INT, "GR-LINE", arg1, th);
+    if (!integerp(arg2))
+	error(NOT_INT, "GR-LINE", arg2, th);
+    if (!integerp(arg3))
+	error(NOT_INT, "GR-LINE", arg3, th);
+    if (!integerp(arg4))
+	error(NOT_INT, "GR-LINE", arg4, th);
+    if (!symbolp(arg5))
+	error(NOT_SYM, "GR-LINE", arg5, th);
+
+    fb_draw_line(GET_INT(arg1), GET_INT(arg2), GET_INT(arg3),
+		 GET_INT(arg4), color_to_number(arg5));
+    return (T);
+}
