@@ -9,10 +9,6 @@
 #include <pthread.h>
 #include <time.h>
 #include "eisl.h"
-#include "compat/nana_stubs.h"
-#include "mem.h"
-#include "except.h"
-#include "fmt.h"
 
 #define DBG_PRINTF(msg,arg)     if(gbc_flag) printf(msg,arg)
 
@@ -46,7 +42,7 @@ void gbc_sequence(void)
 {
     int addr, i, j;
 
-    DBG_PRINTF("enter  concurrent M&S-GC free=%d\n", fc[0]);
+    DBG_PRINTF("enter  M&S-GC free=%d\n", fc[0]);
 
     /* mark hash table */
     for (i = 0; i < HASHTBSIZE; i++)
@@ -99,7 +95,7 @@ void gbc_sequence(void)
     /* mark generic_list */
     mark_cell(generic_list);
 
-    /* mard setf_list. generic functions related setf */
+    /* mark setf_list. generic functions related setf */
     mark_cell(setf_list);
 
 
@@ -143,7 +139,7 @@ void gbc_sequence(void)
 
     }
 
-    DBG_PRINTF("exit   concurrent M&S-GC free=%d\n", fc[0]);
+    DBG_PRINTF("exit   M&S-GC free=%d\n", fc[0]);
 
 }
 
@@ -226,7 +222,7 @@ void mark_cell(int addr)
 	mark_cell(GET_AUX(addr));
 	return;
     default:
-	IP(false, "mark_cell tag switch default action");
+	return;
     }
 }
 
@@ -234,13 +230,16 @@ void mark_cell(int addr)
 void clr_cell(int addr)
 {
     if (IS_VECTOR(addr) || IS_ARRAY(addr)) {
-	if (heap[addr].val.car.dyna_vec)
-	    FREE(heap[addr].val.car.dyna_vec);
+	if (heap[addr].val.car.dyna_vec != NULL)
+	    free(heap[addr].val.car.dyna_vec);
+	heap[addr].val.car.dyna_vec = NULL;
     }
 
     SET_TAG(addr, EMP);
-    if (heap[addr].name)
-	FREE(heap[addr].name);
+    if (heap[addr].name != NULL) {
+	free(heap[addr].name);
+	heap[addr].name = NULL;
+    }
     SET_CAR(addr, 0);
     SET_CDR(addr, 0);
     SET_AUX(addr, 0);
@@ -257,12 +256,12 @@ int check_gbc(int th)
 	    ctrl_c_flag = 0;
 	    printf("ctrl+C\n");
 	    fflush(stdout);
-	    RAISE(Exit_Thread);
+	    longjmp(thread_buf, 1);
 	} else {
 	    ctrl_c_flag = 0;
 	    printf("ctrl+C\n");
 	    fflush(stdout);
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
     }
 
