@@ -13,8 +13,6 @@
 #include <stdlib.h>
 #include <dlfcn.h>
 #include "eisl.h"
-#include "mem.h"
-#include "str.h"
 
 
 static void *hmod;
@@ -51,15 +49,15 @@ void dynamic_link(int x)
     initdeftfunc_t init_deftfunc;
     voidfunc_t init_tfunctions, init_declare;
 
-    if (Str_chr(GET_NAME(x), 1, 0, '/') != 0) {
-	str = Str_dup(GET_NAME(x), 1, 0, 1);
+    if (eisl_strchr(GET_NAME(x), '/') != 0) {
+	str = eisl_strdup(GET_NAME(x));
     } else {
-	str = Str_cat("./", 1, 0, GET_NAME(x), 1, 0);
+	str = eisl_strcat("./", GET_NAME(x));
     }
 
     hmod = dlopen(str, RTLD_LAZY);
 
-    FREE(str);
+    free(str);
     if (hmod == NULL)
 	error(ILLEGAL_ARGS, "load", x, 0);
 
@@ -466,6 +464,6 @@ double get_flt(int x)
 
 int jump_to_repl()
 {
-    RAISE(Restart_Repl);
+    longjmp(repl_buf, 1);
     return (0);
 }
