@@ -43,11 +43,8 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include "eisl.h"
-#include "mem.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
-#include "long.h"
+
+
 
 
 /* pointer */
@@ -189,15 +186,14 @@ int big_pt1 = BIGNUM_PARMA;	/* pointer of parmanent bignum */
 
 
 /* longjmp control and etc */
-Except_T Restart_Repl = { "Restart REPL" },
-    Exit_Interp = { "Exit interpreter" };
 jmp_buf block_buf[CTRLSTK];
 jmp_buf catch_buf[CTRLSTK];
 jmp_buf cont_buf;
-Except_T Ignored_Error = { "Ignored error" };	/* for ignore-errors */
-Except_T Exit_Thread = { "Exit thread" };	/* for Multi-thread */
-Except_T Exit_Process = { "Exit Process" };	/* for Multi-process */
-Except_T Exit_Network = { "Exit Network" };	/* for Distributed parallel */
+jmp_buf error_buf;
+jmp_buf repl_buf;
+jmp_buf thread_buf;
+jmp_buf process_buf;
+jmp_buf network_buf;
 
 int signal_condition_x;
 int signal_condition_y;
@@ -345,8 +341,8 @@ static void usage(void)
 static inline void maybe_greet(void)
 {
     if (greeting_flag)
-	Fmt_print("Easy-ISLisp Ver%1.2f [%dM cells]\n", VERSION,
-		  cell_size / 1000000);
+	printf("Easy-ISLisp Ver%1.2f [%dM cells]\n", VERSION,
+	       cell_size / 1000000);
 
     greeting_flag = false;
 }
@@ -359,9 +355,8 @@ static inline void disable_repl_flag(void)
 
 int main(int argc, char *argv[])
 {
-    int errret;
+    int errret, ret;
 
-    Fmt_register('D', cvt_D);
     if (setupterm((char *) 0, 1, &errret) == ERR ||
 	key_up == NULL || key_down == NULL ||
 	key_right == NULL || key_left == NULL) {
@@ -407,7 +402,8 @@ int main(int argc, char *argv[])
 
     /* handle command line options */
     option_flag = true;
-    TRY {
+    ret = setjmp(repl_buf);
+    if (ret == 0) {
 	if (access("startup.lsp", R_OK) == 0)
 	    f_load(list1(make_str("startup.lsp")), 0);
 
@@ -426,12 +422,12 @@ int main(int argc, char *argv[])
 	    case 'c':
 		str = library_file("compiler.lsp");
 		f_load(list1(make_str(str)), 0);
-		FREE(str);
+		free(str);
 		break;
 	    case 'f':
 		str = library_file("formatter.lsp");
 		f_load(list1(make_str(str)), 0);
-		FREE(str);
+		free(str);
 		break;
 	    case 's':
 		if (access(optarg, R_OK) == -1) {
@@ -462,10 +458,10 @@ int main(int argc, char *argv[])
 		init_creceiver();
 		str = library_file("compiler.lsp");
 		f_load(list1(make_str(str)), 0);
-		FREE(str);
+		free(str);
 		break;
 	    case 'v':
-		Fmt_print("Easy-ISLisp1 Ver%1.2f\n", VERSION);
+		printf("Easy-ISLisp1 Ver%1.2f\n", VERSION);
 		exit(EXIT_SUCCESS);
 	    case 'h':
 		usage();
@@ -481,9 +477,9 @@ int main(int argc, char *argv[])
 	    f_load(list1(make_str(script_arg)), 0);
 	    exit(EXIT_SUCCESS);
 	}
+    } else if (ret == 1) {
+	exit(EXIT_FAILURE);
     }
-    EXCEPT(Restart_Repl) exit(EXIT_FAILURE);
-    END_TRY;
 
     option_flag = false;
 
@@ -492,49 +488,55 @@ int main(int argc, char *argv[])
     volatile bool quit = false;
     do {
 	maybe_greet();
-	TRY while (1) {
-	    init_pointer();
-	    if (!process_flag && !child_flag) {
-		fputs("> ", stdout);
-		print(eval(sread(), 0));
-		putchar('\n');
-	    } else if (process_flag) {
-		TRY print(eval(sread(), 0));
-		putchar('\n');
-		fflush(stdout);
-		union sigval value;
-		value.sival_int = (int) process_num;
-		sigqueue(getppid(), SIGRTMIN, value);
-		EXCEPT(Exit_Process);
-		END_TRY;
-	    } else if (child_flag) {
-		pthread_mutex_lock(&mutex2);
-		while (!child_buffer_ready) {
-		    pthread_cond_wait(&md_cond, &mutex2);
+	ret = setjmp(repl_buf);
+	if (ret == 0) {
+	    while (1) {
+		init_pointer();
+		if (!process_flag && !child_flag) {
+		    fputs("> ", stdout);
+		    print(eval(sread(), 0));
+		    putchar('\n');
+		} else if (process_flag) {
+		    ret = setjmp(process_buf);
+		    if (ret == 0) {
+			print(eval(sread(), 0));
+			putchar('\n');
+			fflush(stdout);
+			union sigval value;
+			value.sival_int = (int) process_num;
+			sigqueue(getppid(), SIGRTMIN, value);
+		    } else if (ret == 1) {
+		    }
+		} else if (child_flag) {
+		    pthread_mutex_lock(&mutex2);
+		    while (!child_buffer_ready) {
+			pthread_cond_wait(&md_cond, &mutex2);
+		    }
+		    child_buffer_ready = 0;
+		    pthread_mutex_unlock(&mutex2);
+		    int exp, res;
+		    exp = str_to_sexp(receive_from_parent());
+		    printf("receive_from_parent ");
+		    print(exp);
+		    printf("\n");
+		    fflush(stdout);
+		    ret = setjmp(network_buf);
+		    if (ret == 0) {
+			res = eval(exp, 0);
+			printf("send_to_parent ");
+			print(res);
+			printf("\n");
+			fflush(stdout);
+			send_to_parent(sexp_to_str(res));
+		    } else if (ret == 1) {
+		    }
 		}
-		child_buffer_ready = 0;
-		pthread_mutex_unlock(&mutex2);
-		int exp, res;
-		exp = str_to_sexp(receive_from_parent());
-		printf("receive_from_parent ");
-		print(exp);
-		printf("\n");
-		fflush(stdout);
-		TRY res = eval(exp, 0);
-		printf("send_to_parent ");
-		print(res);
-		printf("\n");
-		fflush(stdout);
-		send_to_parent(sexp_to_str(res));
-		EXCEPT(Exit_Network);
-		END_TRY;
-	    }
 
-	    if (redef_flag)
-		redef_generic();
-	}
-	EXCEPT(Restart_Repl);
-	EXCEPT(Exit_Interp) {
+		if (redef_flag)
+		    redef_generic();
+	    }
+	} else if (ret == 1) {
+	} else if (ret == 2) {
 	    quit = true;
 	    exit_thread();
 	    close_socket();
@@ -545,7 +547,6 @@ int main(int argc, char *argv[])
 		    error(SYSTEM_ERR, "dp-halt shatdown", NIL, 0);
 	    }
 	}
-	END_TRY;
     }
     while (!quit);
 }
@@ -555,10 +556,10 @@ char *library_file(const char *basename)
     char *prefix;
 
     if ((prefix = getenv("EASY_ISLISP")) != NULL) {
-	return Str_catv(prefix, 1, 0, "/library/", 1, 0, basename, 1, 0,
-			NULL);
+	return eisl_strcatv(prefix, "/library/", basename,
+			    (const char *) NULL);
     }
-    return Str_cat(STRQUOTE(SHAREDIR) "/library/", 1, 0, basename, 1, 0);
+    return eisl_strcat(STRQUOTE(SHAREDIR) "/library/", basename);
 }
 
 void init_dp(void)
@@ -663,7 +664,7 @@ int readc(void)
 	if (!script_flag && input_stream == standard_input && c == EOF) {
 	    /* ctrl+D and not script-mode quit system */
 	    putchar('\n');
-	    RAISE(Exit_Interp);
+	    longjmp(repl_buf, 2);
 	} else if (script_flag)
 	    /* on script-mode only retrun c */
 	    return (c);
@@ -699,7 +700,7 @@ void init_stok()
     stok.flag = GO;
     stok.type = OTHER;
     stok.bufsize = BUFSIZE;
-    stok.buf = ALLOC(BUFSIZE * sizeof(char));
+    stok.buf = malloc(BUFSIZE * sizeof(char));
     stok.buf[0] = '\0';
 }
 
@@ -707,7 +708,12 @@ void set_stok_buf(int index, char c)
 {
     if (index >= stok.bufsize) {
 	int new_bufsize = index + 1;
-	RESIZE(stok.buf, sizeof(char) * new_bufsize);
+	char *new_buf = (char *) realloc(stok.buf, new_bufsize);
+	if (!new_buf) {
+	    fprintf(stderr, "Memory allocation failed\n");
+	    exit(1);
+	}
+	stok.buf = new_buf;
 	stok.bufsize = new_bufsize;
 
     }
@@ -719,7 +725,12 @@ void replace_stok_buf(char *str)
 {
     if (strlen(str) > (size_t) stok.bufsize) {
 	int new_bufsize = strlen(str);
-	RESIZE(stok.buf, sizeof(char) * new_bufsize);
+	char *new_buf = (char *) realloc(stok.buf, new_bufsize);
+	if (!new_buf) {
+	    fprintf(stderr, "Memory allocation failed\n");
+	    exit(1);
+	}
+	stok.buf = new_buf;
 	stok.bufsize = new_bufsize;
     }
     strcpy(stok.buf, str);
@@ -1576,10 +1587,10 @@ void print(int addr)
 void print_int(int addr)
 {
     if (GET_OPT(output_stream) != EISL_OUTSTR)
-	Fmt_fprint(GET_PORT(output_stream), "%d", GET_INT(addr));
+	fprintf(GET_PORT(output_stream), "%d", GET_INT(addr));
     else {
 	char str[SHORT_STRSIZE];
-	Fmt_sfmt(str, SHORT_STRSIZE, "%d", GET_INT(addr));
+	sprintf(str, "%d", GET_INT(addr));
 	append_str(output_stream, str);
     }
 }
@@ -1606,15 +1617,11 @@ void print_long(int addr)
 #ifdef __rpi__
 	fprintf(GET_PORT(output_stream), "%lld", GET_LONG(addr));
 #else
-	Fmt_fprint(GET_PORT(output_stream), "%D", GET_LONG(addr));
+	fprintf(GET_PORT(output_stream), "%lld", GET_LONG(addr));
 #endif
     } else {
 	char str[SHORT_STRSIZE];
-#ifdef __rpi__
-	sprintf(str, SHORT_STRSIZE, "%lld", GET_LONG(addr));
-#else
-	Fmt_sfmt(str, SHORT_STRSIZE, "%D", GET_LONG(addr));
-#endif
+	sprintf(str, "%lld", GET_LONG(addr));
 	append_str(output_stream, str);
     }
 }
@@ -1670,10 +1677,10 @@ void print_arr(int x)
 	ls = cons(GET_VEC_ELT(x, i), ls);
     ls = reverse(ls);
     if (GET_OPT(output_stream) != EISL_INSTR)
-	Fmt_fprint(GET_PORT(output_stream), "#%da", dim);
+	fprintf(GET_PORT(output_stream), "#%da", dim);
     else {
 	char str[SHORT_STRSIZE];
-	Fmt_sfmt(str, SHORT_STRSIZE, "#%da", dim);
+	sprintf(str, "#%da", dim);
 	append_str(output_stream, str);
     }
     if (dim == 0)
@@ -1686,9 +1693,9 @@ void print_arr(int x)
 void print_str(int addr)
 {
     if (GET_OPT(output_stream) != EISL_OUTSTR) {
-	Fmt_fprint(GET_PORT(output_stream), "\"%s\"", GET_NAME(addr));
+	fprintf(GET_PORT(output_stream), "\"%s\"", GET_NAME(addr));
     } else {
-	Fmt_sfmt(stream_str, STRSIZE, "\"%s\"", GET_NAME(addr));
+	sprintf(stream_str, "\"%s\"", GET_NAME(addr));
 	append_str(output_stream, stream_str);
     }
 }
@@ -1805,9 +1812,9 @@ void print_obj(const char *str)
 void print_class(int addr)
 {
     if (GET_OPT(output_stream) != EISL_OUTSTR)
-	Fmt_fprint(GET_PORT(output_stream), "<class %s>", GET_NAME(addr));
+	fprintf(GET_PORT(output_stream), "<class %s>", GET_NAME(addr));
     else {
-	Fmt_sfmt(stream_str, STRSIZE, "<class %s>", GET_NAME(addr));
+	sprintf(stream_str, "<class %s>", GET_NAME(addr));
 	append_str(output_stream, stream_str);
     }
 }
@@ -1824,9 +1831,9 @@ void print_stm(int addr)
 	name = GET_NAME(addr);
     }
     if (GET_OPT(output_stream) != EISL_OUTSTR)
-	Fmt_fprint(GET_PORT(output_stream), "<stream %s>", name);
+	fprintf(GET_PORT(output_stream), "<stream %s>", name);
     else {
-	Fmt_sfmt(GET_NAME(output_stream), STRSIZE, "<stream %s>", name);
+	sprintf(GET_NAME(output_stream), "<stream %s>", name);
 	append_str(output_stream, stream_str);
     }
 }
@@ -2388,9 +2395,10 @@ void bind_macro(char *name, int addr)
     SET_CDR(val1, 0);
     val2 = freshcell();
     SET_TAG(val2, MACRO);
-    TRY heap[val2].name = Str_dup(name, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "makemacro", NIL, 0);
-    END_TRY;
+    heap[val2].name = eisl_strdup(name);
+    if (heap[val2].name == NULL) {
+	error(MALLOC_OVERF, "makemacro", NIL, 0);
+    }
     SET_CAR(val2, val1);
     SET_CDR(val2, 0);
     SET_AUX(val2, cfunction);	/* class */
