@@ -9,11 +9,6 @@
 #include <pthread.h>
 #include <sys/time.h>
 #include "eisl.h"
-#include "mem.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
-#include "text.h"
 
 
 /* multi thread parallel functions */
@@ -60,6 +55,7 @@ int eval_para(int arg)
 
 void *parallel(void *arg)
 {
+    int ret;
     int num = *(int *) arg;
 
     while (1) {
@@ -70,9 +66,11 @@ void *parallel(void *arg)
 	    goto exit;
 
 	ep[num] = ep[0];
-	TRY para_output[num] = eval(para_input[num], num);
-	EXCEPT(Exit_Thread);
-	END_TRY;
+	ret = setjmp(thread_buf);
+	if (ret == 0) {
+	    para_output[num] = eval(para_input[num], num);
+	} else if (ret == 1) {
+	}
 	mt_enqueue(num);
 	if (mt_queue_pt == mt_queue_num) {
 	    pthread_mutex_lock(&mutex);
@@ -291,7 +289,7 @@ int f_mt_call(int arglist, int th)
 
 int f_mt_exec(int arglist, int th)
 {
-    int temp, i, num[PARASIZE];
+    int temp, i, num[PARASIZE] = { 0 };
 
     if (length(arglist) == 0)
 	error(WRONG_ARGS, "mt-exec", arglist, th);
@@ -408,10 +406,10 @@ int sexp_to_str(int x)
     char *str;
 
     res = make_stm(stdout, EISL_OUTSTR, NULL);
-    TRY str = (char *) ALLOC(STRSIZE);
-    EXCEPT(Mem_Failed)
+    str = (char *) malloc(STRSIZE);
+    if (str == NULL) {
 	error(MALLOC_OVERF, "create-string-output-stream", NIL, 0);
-    END_TRY;
+    }
     heap[res].name = str;
     heap[res].name[0] = '\0';
 
@@ -579,10 +577,10 @@ int str_to_sexp(int x)
     int stm, save, res;
 
     stm = make_stm(stdin, EISL_INSTR, NULL);
-    TRY heap[stm].name = Str_dup(GET_NAME(x), 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "create-string-input-stream",
-			     NIL, 0);
-    END_TRY;
+    heap[stm].name = eisl_strdup(GET_NAME(x));
+    if (heap[stm].name == NULL) {
+	error(MALLOC_OVERF, "create-string-input-stream", NIL, 0);
+    }
 
     save = input_stream;
     input_stream = stm;
@@ -735,6 +733,7 @@ int f_mp_or(int arglist, int th)
 	temp = cdr(temp);
 	i++;
     }
+    res = NIL;
     for (i = 0; i < n; i++) {
 	res = str_to_sexp(read_from_pipe_part(n));
 	if (res != NIL)
@@ -833,7 +832,7 @@ int f_dp_close(int arglist, int th)
 	shutdown(parent_sockfd[1], SHUT_RDWR);
 	close(parent_sockfd[0]);
 	close(parent_sockfd[1]);
-	RAISE(Exit_Interp);
+	longjmp(repl_buf, 2);
     }
 
     receiver_exit_flag = 1;
@@ -867,7 +866,7 @@ int f_dp_halt(int arglist, int th)
     if (child_flag) {
 	printf("Easy-ISLisp exit network mode.\n");
 	shutdown_flag = true;
-	RAISE(Exit_Interp);
+	longjmp(repl_buf, 2);
     }
 
     receiver_exit_flag = 1;
@@ -1076,9 +1075,10 @@ int f_dp_eval(int arglist, int th __unused)
     arg1 = car(arglist);
     arg2 = cadr(arglist);
     i = GET_INT(arg1);
-    if (i >= child_num || i < 0)
+    if (i >= child_num || i < 0) {
 	error(WRONG_ARGS, "dp-eval", arg1, 0);
-
+	return (0);
+    }
     memset(parent_buffer[i], 0, sizeof(parent_buffer[i]));
     send_to_child(GET_INT(arg1), sexp_to_str(arg2));
     while (parent_buffer[i][0] == 0) {
@@ -1251,7 +1251,7 @@ int wait_all(int m)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < m; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1267,9 +1267,11 @@ int wait_and(int m)
 {
     int i, j, res, result[PARASIZE];
 
-    for (i = 0; i < m; i++)
+    for (i = 0; i < m; i++) {
 	result[i] = 0;
+    }
 
+    res = NIL;
     while (!all_received(result, m)) {
 	if (ctrl_c_flag == 1) {
 	    for (i = 0; i < m; i++) {
@@ -1278,7 +1280,7 @@ int wait_and(int m)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < m; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1322,7 +1324,7 @@ int wait_or(int m)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 
 	for (i = 0; i < m; i++) {
@@ -1391,7 +1393,7 @@ int f_dp_call(int arglist, int th)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < n; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1448,7 +1450,7 @@ int f_dp_exec(int arglist, int th)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < n; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1498,7 +1500,7 @@ int f_dp_and(int arglist, int th)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < n; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1564,7 +1566,7 @@ int f_dp_or(int arglist, int th)
 	    }
 	    printf("ctrl+C\n");
 	    ctrl_c_flag = 0;
-	    RAISE(Restart_Repl);
+	    longjmp(repl_buf, 1);
 	}
 	for (i = 0; i < n; i++) {
 	    if (parent_buffer[i][0] != 0 && result[i] == 0) {
@@ -1633,6 +1635,9 @@ void *preceiver(void *arg)
 	m = read(child_sockfd[n], sub_buffer, sizeof(sub_buffer));
 	if (m < 0 && !receiver_exit_flag) {
 	    error(SYSTEM_ERR, "receive from child", make_int(n), 0);
+	    return (NULL);
+	} else if (m == 0) {
+	    return (NULL);
 	}
 	//print_ascii(sub_buffer);printf("m=%d",m);fflush(stdout);
 	strcat(buffer, sub_buffer);
@@ -1703,6 +1708,9 @@ void *creceiver(void *arg)
 	n = read(parent_sockfd[1], sub_buffer, sizeof(sub_buffer));
 	if (n < 0) {
 	    error(SYSTEM_ERR, "*creceiver", NIL, 0);
+	    return (NULL);
+	} else if (n == 0) {
+	    return (NULL);
 	}
 
 	strcat(buffer, sub_buffer);
