@@ -13,10 +13,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include "eisl.h"
-#include "compat/nana_stubs.h"
-#include "fmt.h"
-#include "str.h"
-#include "mem.h"
+
 
 
 #define TAGBODY_LEN_MAX 100
@@ -1924,14 +1921,17 @@ int f_defmethod(int arglist, int th)
 
 int f_ignore_errors(int arglist, int th)
 {
-    volatile int res, save1, save2;
+    volatile int res, save1, save2, ret;
 
     ignore_flag = true;
     save1 = input_stream;
     save2 = output_stream;
-    TRY res = f_progn(arglist, th);
-    ELSE res = NIL;
-    END_TRY;
+    ret = setjmp(error_buf);
+    if (ret == 0) {
+	res = f_progn(arglist, th);
+    } else if (ret == 1) {
+	res = NIL;
+    }
     ignore_flag = false;
     try_flag = false;
     input_stream = save1;
@@ -1972,9 +1972,9 @@ int f_with_open_input_file(int arglist, int th)
 	return NIL;
     }
     if (n == 2)
-	val = make_stm(port, EISL_INPUT, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_INPUT, eisl_strdup(fname));
     else
-	val = make_stm(port, EISL_INPUT_BIN, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_INPUT_BIN, eisl_strdup(fname));
     ep1 = ep[th];
     add_lex_env(sym, val, th);
     res = f_progn(arg2, th);
@@ -2011,9 +2011,9 @@ int f_with_open_output_file(int arglist, int th)
 	return NIL;
     }
     if (n == 2)
-	val = make_stm(port, EISL_OUTPUT, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_OUTPUT, eisl_strdup(fname));
     else
-	val = make_stm(port, EISL_OUTPUT_BIN, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_OUTPUT_BIN, eisl_strdup(fname));
     ep1 = ep[th];
     add_lex_env(sym, val, th);
     res = f_progn(arg2, th);
@@ -2050,9 +2050,9 @@ int f_with_open_io_file(int arglist, int th)
 	return NIL;
     }
     if (n == 2)
-	val = make_stm(port, EISL_INOUT, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_INOUT, eisl_strdup(fname));
     else
-	val = make_stm(port, EISL_INOUT_BIN, Str_dup(fname, 1, 0, 1));
+	val = make_stm(port, EISL_INOUT_BIN, eisl_strdup(fname));
     ep1 = ep[th];
     add_lex_env(sym, val, th);
     res = f_progn(arg2, th);
@@ -2159,11 +2159,7 @@ int convert(int arg1, int arg2)
 	} else if (GET_AUX(arg2) == cfloat) {
 	    return (exact_to_inexact(arg1));
 	} else if (GET_AUX(arg2) == cstring) {
-#ifdef __rpi__
 	    sprintf(str, "%lld", GET_LONG(arg1));
-#else
-	    Fmt_sfmt(str, SHORT_STRSIZE, "%D", GET_LONG(arg1));
-#endif
 	    return (make_str(str));
 	}
 	break;
@@ -2185,7 +2181,7 @@ int convert(int arg1, int arg2)
 	} else if (GET_AUX(arg2) == cfloat) {
 	    return (exact_to_inexact(arg1));
 	} else if (GET_AUX(arg2) == cstring) {
-	    Fmt_sfmt(str, SHORT_STRSIZE, "%d", GET_INT(arg1));
+	    sprintf(str, "%d", GET_INT(arg1));
 	    return (make_str(str));
 	}
 	break;
@@ -2270,7 +2266,7 @@ int convert(int arg1, int arg2)
 	}
 	break;
     default:
-	VL(("convert tag switch default action"));
+	error(OUT_OF_DOMAIN, "convert", arg1, 0);
     }
     error(OUT_OF_DOMAIN, "convert", arg1, 0);
     return (UNDEF);
